#include "StdAfx.h"
#include "UIChatFileElement.h"
#include "../DuiLib/Control/UIRichEdit.h"
#include "ChatDlg.h"
using namespace FileTransferState;


#define DEF_FILE_MSG_BUBBLE_TOP_Y		        (20)
#define DEF_FILE_MSG_BUBBLE_USERNAME_TOP_Y		(8)

#define DEF_FILE_MSG_BUBBLE_ARROR_X			(6)
#define DEF_FILE_MSG_PLAIN_ARROR_X			(7)

//---------------------------------------
// implementation of UIChatFileElement
//


UIChatFileElement::UIChatFileElement( CChatItem* pChatItem, IUIElementResouce* pResouce) : UIElement(pChatItem, pResouce)
{
	_enum_file_transfer_state transfer_state = FTS_NONE;
	CChatFileItem* pFileItem = dynamic_cast<CChatFileItem*>(pChatItem);

	ASSERT(pFileItem);
	if(pFileItem)
	{
		if(pFileItem->GetTranslateFileInfo()->IsUploadJob())
		{
			TransferStatus status = pFileItem->GetTranslateFileInfo()->GetStatus();
			if(status == sUploadSuccess)
			{
				transfer_state = send_complete;
			}
			else if(status == sUploadCancle)
			{
				transfer_state = send_cancel;
			}
			else if(status == sUploadError)
			{
				transfer_state = send_error;
			}
			else
			{
				transfer_state = send_init;
			}
		}
		else
		{
			TransferStatus status = pFileItem->GetTranslateFileInfo()->GetStatus();
			if(status == sRecvSuccess)
			{
			    transfer_state = recv_complete;
			}
			else if(status == sRecvCancle)
			{
				transfer_state = recv_cancel;
			}
			else if(status == sRecvError)
			{
				transfer_state = recv_error;
			}
			else
			{
				transfer_state = recv_init;
			}
		}
	}
	m_nProgress = 0;
	m_pStateManager  = new CFTStateManager(this);
	ASSERT(m_pStateManager);
	m_pStateManager->SetState(transfer_state);
}

UIChatFileElement::~UIChatFileElement()
{
	if (m_pStateManager)
	{
		delete m_pStateManager;
		m_pStateManager = NULL;
	}
}

CChatFileItem* UIChatFileElement::GetChatFileItem()
{
	CChatFileItem* pFileChatItem = dynamic_cast<CChatFileItem*>(m_pChatItem);
	ASSERT(pFileChatItem);
	return pFileChatItem;
}

IUIElementResouce* UIChatFileElement::GetElementResource()
{
	return __super::m_pResouce;
}

void UIChatFileElement::DrawRetryStatus(Graphics* graph, CRect rcPS)
{
	if (!m_pChatItem->IsFailedMsg() || rcPS.IsRectEmpty() == TRUE)
	{
		return;
	}

	CRect rcView = m_pResouce->GetViewRect();

	CRect rcReSend = rcPS;

	Image* pReSendImage = m_pResouce->GetSendStatusImage(4);
	if (pReSendImage)
	{
		CRect rcImage(rcReSend.left, rcReSend.top,
			rcReSend.left + pReSendImage->GetWidth(),
			rcReSend.top + pReSendImage->GetHeight());

		InterpolationMode interpolationMode = graph->GetInterpolationMode();
		PixelOffsetMode pixelOffsetMode = graph->GetPixelOffsetMode();
		graph->SetInterpolationMode(InterpolationModeNearestNeighbor);
		graph->SetPixelOffsetMode(PixelOffsetModeHalf);

		DrawImageToDest(graph, pReSendImage, &rcView, &rcImage, NULL);

		graph->SetInterpolationMode(interpolationMode);
		graph->SetPixelOffsetMode(pixelOffsetMode);
	}
}

BOOL UIChatFileElement::Draw( Graphics *graph )
{
	CPoint ptZero(0, 0);
	CPoint ptOffset = ptZero - m_pResouce->GetOffSetPoint();
	CRect rcClient = m_pResouce->GetViewRect();

	ImageAttributes ImgAttr;
	//if (CHAT_BUBBLE == GET_CONFIG()->GetChatDisplayType())
	if (CHAT_BUBBLE == m_pResouce->GetChatDisplayType())
	{
		if (m_dwFlags & DEF_CHAT_MSG_DATE)
		{
			DrawDate(graph);
		}
		if ((m_dwFlags & DEF_CHAT_MSG_USERNAME) && !(m_dwFlags & DEF_CHAT_MSG_ISMINE))
		{
			DrawUserName(graph);
			DrawUserImage(graph);
			DrawTime(graph);
		}
		else
		{
			if (m_dwFlags & DEF_CHAT_MSG_TIME)
			{	
				DrawTime(graph);
			}
			else if (m_dwFlags & DEF_CHAT_MSG_RESEND)
			{
				//CPoint ptZero(0, 0);
				//CPoint ptOffset = ptZero - m_pResouce->GetOffSetPoint();
				CRect rcReSend = m_rcReSendImg;
				rcReSend.OffsetRect(ptOffset);
				DrawRetryStatus(graph,rcReSend);
			}
		}
		//CRect rcItem = m_rcItem;
		//Pen pen(Color::Red);
		//RectF rcf(rcItem.left, rcItem.top, rcItem.Width(), rcItem.Height());
		//graph->DrawRectangle(&pen, rcf);
		return m_pStateManager->DrawItems(graph);
	}
	//else if (CHAT_PLAIN == GET_CONFIG()->GetChatDisplayType())
	else if (CHAT_PLAIN == m_pResouce->GetChatDisplayType())
	{
		//if (m_dwFlags & DEF_CHAT_MSG_USERNAME)
		//{
			DrawUserName(graph);
			DrawUserImage(graph);
		//}
		DrawTime(graph);
		return m_pStateManager->DrawItems(graph);
	}

	return FALSE;
}

CSize UIChatFileElement::_CalcReSendStatus( const CPoint& ptPos, CSize& sz )
{
	m_rcReSendImg.SetRectEmpty();
	CSize szRet(0, 0);
	int nWidth = (m_pResouce->GetSendStatusImage(0)->GetWidth() + 
		2 + 
		m_pResouce->GetSendStatusImage(2)->GetWidth());

	m_rcReSendImg.bottom = m_rcBubble.bottom - BUBBLE_TIME_MARGIN_Y;
	m_rcReSendImg.top = m_rcReSendImg.bottom - m_pResouce->GetSendStatusImage(0)->GetHeight();

	m_rcReSendImg.right = m_rcBubble.left - BUBBLE_TIME_MARGIN_X;
	m_rcReSendImg.left = m_rcReSendImg.right - nWidth;
	m_rcReSendImg.OffsetRect(ptPos.x,ptPos.y);

	return CSize(0,0);
}

CSize UIChatFileElement::CalcSize( const CPoint& ptPos, DWORD dwFlags )
{
	m_dwFlags = dwFlags;
	
	CPoint ptOffset = ptPos;
	CSize sz(0, 0);

	//bubble
	//if (CHAT_BUBBLE == GET_CONFIG()->GetChatDisplayType())
	if (CHAT_BUBBLE == m_pResouce->GetChatDisplayType())
	{
		if(dwFlags & DEF_CHAT_MSG_DATE)
		{
			CalcDateSize(ptOffset, sz);
			ptOffset.y += sz.cy;
		}

		int nTop = DEF_FILE_MSG_BUBBLE_TOP_Y;
		if((dwFlags & DEF_CHAT_MSG_USERNAME) && !(dwFlags & DEF_CHAT_MSG_ISMINE))
		{
			CSize szUserName(0,0);
			CSize szUserImage(0, 0);
			CalcUserImage(ptOffset,szUserImage);
			CalcUserName(ptOffset,szUserName);
			

			ptOffset.y += szUserName.cy;
			sz.cx = max(sz.cx, szUserName.cx);
			sz.cy += szUserName.cy;

			nTop = DEF_FILE_MSG_BUBBLE_USERNAME_TOP_Y;
		}
		else
		{			
			/*if (dwFlags & DEF_CHAT_MSG_TIME)
			{
				CSize szTime(0, 0);
				CalcTime(ptOffset, szTime, dwFlags);
				ptOffset.y += szTime.cy;
				sz.cx = max(sz.cx, szTime.cx);
				sz.cy += szTime.cy;
				nTop = DEF_FILE_MSG_BUBBLE_USERNAME_TOP_Y;
			}*/
		}
	
		ptOffset.y += nTop;
		ptOffset.x += DEF_ITEM_POS_LEFT_REL_X + _GetUserImageWidth() + DEF_FILE_MSG_BUBBLE_ARROR_X;
		sz.cy += nTop;

		CSize szFTInfo = m_pStateManager->CalcSize(ptOffset, dwFlags, m_rcBubble);
		sz.cx = max(sz.cx, szFTInfo.cx);
		sz.cy += szFTInfo.cy;


		if (dwFlags & DEF_CHAT_MSG_TIME)
		{
			CSize szTime(0, 0);
			CalcTime(ptOffset, szTime, dwFlags);
			//ptOffset.y += szTime.cy;
			//sz.cy += szTime.cy;
			sz.cx = max(szTime.cx, sz.cx);
		}
		else if (dwFlags & DEF_CHAT_MSG_RESEND)
		{
			m_rcReSendImg.SetRectEmpty();
			int nWidth = (m_pResouce->GetSendStatusImage(0)->GetWidth() + 
				2 + 
				m_pResouce->GetSendStatusImage(2)->GetWidth());

			m_rcReSendImg.bottom = m_rcBubble.bottom - BUBBLE_TIME_MARGIN_Y;
			m_rcReSendImg.top = m_rcReSendImg.bottom - m_pResouce->GetSendStatusImage(0)->GetHeight();

			m_rcReSendImg.right = m_rcBubble.left - BUBBLE_TIME_MARGIN_X;
			m_rcReSendImg.left = m_rcReSendImg.right - nWidth;
		}
	}
	//else if(CHAT_PLAIN == GET_CONFIG()->GetChatDisplayType())
	else if(CHAT_PLAIN == m_pResouce->GetChatDisplayType())
	{
		//if(dwFlags & DEF_CHAT_MSG_USERNAME)
		//{
			CSize szUserName(0, 0);
			CSize szUserImage(0, 0);
			CalcUserImage(ptPos,szUserImage);
			CalcUserName(ptPos, szUserName);
			sz.cx = max(sz.cx, szUserName.cx);
			sz.cy += szUserName.cy;
			ptOffset.y += szUserName.cy;
		//}

		
		CSize szTime(0, 0);
		CalcTime(ptOffset, szTime, dwFlags);
		//ptOffset.y += szTime.cy;
		sz.cx = max(sz.cx, szTime.cx);
		//sz.cy += szTime.cy;

		ptOffset.x += DEF_ITEM_POS_LEFT_REL_X + _GetUserImageWidth() + DEF_FILE_MSG_PLAIN_ARROR_X;

        CRect rcRemark(0, 0, 0, 0);
		CSize szFTInfo = m_pStateManager->CalcSize(ptOffset, dwFlags, rcRemark);
		sz.cx = max(sz.cx, szFTInfo.cx);
		sz.cy += szFTInfo.cy;
	}
	m_rcItem.SetRect(0, 0, m_pResouce->GetViewRect().Width(), sz.cy);
	m_rcItem.OffsetRect(ptPos);

	return sz;
}


//CSize  UIChatFileElement::GetMaxSize(CSize sz_basic, CPoint pt_offset)
//{
//	return CSize(0, 0);// m_pStateManager->GetStatePtr()->GetMaxSize(sz_basic, pt_offset);
//}

//BOOL   UIChatFileElement::DrawItems(CPoint pt_offset, Graphics *graph )
//{
//	return m_pStateManager->GetStatePtr()->DrawItems(graph);
//
//}


void UIChatFileElement::OnLButtonDown(UINT nFlags, CPoint point)
{
	ResetSelectRange();
	m_pStateManager->OnLButtonDown(nFlags, point);
	UIElement::OnLButtonDown(nFlags, point);
}

void UIChatFileElement::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_pStateManager->OnLButtonUp(nFlags, point);

}

BOOL UIChatFileElement::OnMouseMove( UINT nFlags, CPoint point )
{
	BOOL bHandle = UIElement::OnMouseMove(nFlags, point);
	bHandle =  bHandle || m_pStateManager->OnMouseMove(nFlags, point);


	if (MK_LBUTTON == nFlags && m_pResouce->IsMouseSelection() &&
		CHAT_PLAIN == m_pResouce->GetChatDisplayType())
	{
		CPoint ptStart = m_pResouce->GetSelectionStartPoint();
		CPoint ptEnd = point;

		PlainNormalizePoint(ptStart, ptEnd);
		_SetNameSlectedItem(ptStart, ptEnd, FALSE);
		_SetTimeSlectedItem(ptStart, ptEnd, FALSE);
		//m_pResouce->InvalidateElement(this, FALSE);

	}

	return bHandle;

}

void UIChatFileElement::OnTransferStart(TranslateFileInfo& fileInfo)
{
	TRACE(TEXT("OnTransferStart, JobIndex= %d\n"), fileInfo.m_nJobIndex);
	if (fileInfo.IsUploadJob())
	{
		m_pStateManager->SetState(FileTransferState::send_progress);
	}
	else
	{
		m_pStateManager->SetState(FileTransferState::recv_progress);
	}

	m_pChatMainView->UpdateElementStateByPointer((UIElement*)this);
	m_pChatMainView->UpdateView();
}

void UIChatFileElement::OnTransferProgress(TranslateFileInfo& fileInfo, INT nProgress, BOOL bCalcSize)
{
	//TRACE(TEXT("OnTransferProgress, JobIndex= %d, nProgress=%d\n"), fileInfo.m_nJobIndex, nProgress);
	m_nProgress = nProgress;
	m_pChatMainView->UpdateView();
	//m_pResouce->InvalidateElement(this, TRUE);
}

void UIChatFileElement::OnTransferError(TranslateFileInfo& fileInfo, TransferError err)
{
	TRACE(TEXT("OnTransferError, JobIndex= %d, err=%d\n"), fileInfo.m_nJobIndex, err);
	if (err == eUserCancel)
	{
		if (fileInfo.IsUploadJob())
		{
			m_pStateManager->SetState(FileTransferState::send_cancel);
		}
		else
		{
			m_pStateManager->SetState(FileTransferState::recv_cancel);
		}
	}
	else
	{
		if (fileInfo.IsUploadJob())
		{
			m_pStateManager->SetState(FileTransferState::send_error);
		}
		else
		{
			m_pStateManager->SetState(FileTransferState::recv_error);
		}
	}

	m_pChatMainView->UpdateElementStateByPointer((UIElement*)this);
	m_pChatMainView->UpdateView();
}

void UIChatFileElement::OnTransferEnd(TranslateFileInfo& fileInfo, TransferStatus status)
{
	TRACE(TEXT("OnTransferEnd, JobIndex= %d, status=%d\n"), fileInfo.m_nJobIndex, status);
	if (sUploadSuccess == status || sRecvSuccess == status)
	{
		if (fileInfo.IsUploadJob())
		{
			m_pStateManager->SetState(FileTransferState::send_complete);
		}
		else
		{
			m_pStateManager->SetState(FileTransferState::recv_complete);
		}
	}

	m_pChatMainView->UpdateElementStateByPointer((UIElement*)this);
	m_pChatMainView->UpdateView();
}

